/*
 * Program to display pending uucp actions.  Assume HDB uucp.  Originally
 * written by Bill Davidson of General Electric, davidsen@@crdos1.crd.ge.com.
 * Taken from pending.c v1.2, 10/29/90.  Bill's copyright:
 * 
 * Copyright 1988 by Bill Davidsen, all rights reserved
 * May be freely distributed and used providing copyright
 * notices are kept intact.
 *
 * Changes Copyright 1990 Steven C. Simmons (scs@lokkur.dexter.mi.us)
 * All rights reserved.  May be freely distributed and used providing
 * copyright notices are kept intact.
 *
 * $RCSfile: uupend.c,v $	$Revision: 1.4 $
 *
 * $Author: scs $	$Date: 90/11/11 18:25:10 $
 *
 * $State: Exp $	$Locker:  $
 *
 * $Log:	uupend.c,v $
 * Revision 1.4  90/11/11  18:25:10  scs
 * Added full switch processing, -v and -h switch, multiple simultaneous
 * system checks, a number of modules from the Inland Sea library, and
 * lots of general robustness.  Error messages are improved.  Full ANSI
 * and non-ANSI compilation tested.
 * 
 * Revision 1.3  90/11/10  11:56:44  scs
 * Completed basic ansi-fication -- testing needed.
 */


static char	rcsid[] = "$Id: uupend.c,v 1.4 90/11/11 18:25:10 scs Exp $" ;

# include	<stdio.h>
# include	<ctype.h>


#define EOS	'\0'

#ifndef	SPOOLDIR
# define	SPOOLDIR	"/var/spool/uucp"
#endif

#ifndef	MAXPATHLEN
# define	MAXPATHLEN	1024
#endif

#ifndef	MAXNAMELEN
# define	MAXNAMELEN	64
#endif

#ifndef	BUFSIZ
# define	BUFSIZE	256
#endif

#ifdef	__STDC__
extern FILE	*popen( char *c, char *t ) ;
extern FILE	*fopen( char *s, char *m ) ;
extern void	exit( int ) ;
extern int	chdir( char *s ) ;
extern int	perror( char *s ) ;
extern char	*fgets( char *string, int len, FILE *f ) ;
extern int	strlen( char *string ) ;
extern int	fclose( FILE *f ) ;
extern int	pclose( FILE *f ) ;
extern int	fprintf( FILE *f, char *s, ... ) ;
extern int	printf( char *s, ... ) ;
extern char	*sprintf( char *s, char *f, ... ) ;
extern int	sscanf( char *str, char *pat, ... ) ;
extern int	fputs( char *s, FILE *f ) ; 
extern int	strncmp( char *str1, char *str2, int len ) ;
extern void	*malloc( int len ) ;
extern char	*strcpy( char *target, char *source ) ;
extern int	strlen( char *string ) ;
extern char	*strchr(char *string, char c ) ;

#else
extern FILE	*popen() ;
extern FILE	*fopen() ;
extern void	exit() ;
extern int	chdir() ;
extern int	perror() ;
extern char	*fgets() ;
extern int	strlen() ;
extern int	sscanf() ;
extern int	strncmp() ;
extern int	fclose() ;
extern int	pclose() ;
extern int	fputs() ;
extern int	getopt() ;
extern char	*malloc() ;
extern char	*strcpy() ;
extern int	strlen() ;
extern char*	strchr() ;
#endif


typedef	int	bool ;

# define	TRUE	(1)
# define	FALSE	(0)

extern int	errno ;

static bool	version_flag = FALSE ;
static bool	help_flag = FALSE ;

static char	*SystemList[ BUFSIZ ] ;

static char	*ProgName ;

static char	*IDent[] =
{
	"@( # )pending.c v1.2, 10/29/90",
	rcsid,
	"Copyright 1988 by Bill Davidsen, all rights reserved.  May be freely",
	"distributed and used providing copyright notices are kept intact.",
	"Changes Copyright 1990 by Steven C. Simmons, same terms.",
	NULL
} ;


/*
 *  Library function to create new copies of strings.  Creates
 *  exactly the same size offered.  General scheme:
 *
 *  Create a new string.  Return a pointer to it.  Return null
 *  if malloc fails, or if the incoming string is a null pointer.
 *  A zero-length string (ie, '\0' only) is a valid string, so
 *  we'll duplicate it.
 *
 *  History -- this function was once part of SCS standard library.
 *  It has been moved here rather than carry the whole library along.
 */


#ifdef	__STDC__
char	*NewString( char* string )
#else
char	*NewString( string )
register char	*string ;
#endif
{
	register unsigned	len ;
	register char*		new_string ;

	if ( string == NULL )
		return NULL ;
	len = strlen( string ) ;
	if ( NULL == ( new_string = malloc( len + 1 ) ) )
		return NULL ;
	(void) strcpy( new_string, string ) ;
	return new_string ;
}


/*	Module-wide variables for OptionParse	*/

static int	argnum = 1 ;	/* current location in argv */
static int	argchar = 1 ;	/* which character in arg we're looking at */
static char	swflag = '-' ;	/* what character delimits a switch */
static char	extarg = ':' ;	/* flag for extended argument wanted */
static char	errflg = '?' ;	/* what to return on a non-existant switch */
static char	dummy[ 2 ] ;	/* buffer to build error returns */

/*	Global variables for OptionParse	*/

char*	OptionSwitch = NULL ;	/* pointer to return an extended arguement */


/*
 *  OptionParse -- scan the arg list and return next arguement
 *
 *  parameters
 *	argc -- maximum number of arguements
 *	argv -- array of argc pointers to arguements
 *	opts -- list of valid switches
 *
 *  return value (int)
 *	null character if no switch is on the arguement, and
 *		OptionSwitch points to the arg;
 *	valid character if found behind switch, and OptionSwitch
 *		points to extended if requested (NULL if not or
 *		if there is none);
 *	errflg if illegal character found behind switch and
 *		OptionSwitch points to illegal character;
 *	EOF if no args to process.
 *
 *  This function scans the arguement list passed in, starting
 *  from an internal pointer into the list.  Each time it is
 *  called, the internal pointer is reset so the next call will
 *  scan the next arg.
 *
 *  An arguement is first checked to see if it begins with the
 *  switch character.  If it does not, it is assumed to be a
 *  non-switched arguement (eg, a file name).  A pointer to is
 *  is copied into OptionSwitch, and we return a null character.
 *
 *  If it is a switched arg, we return check the first character
 *  following the switch against the opts list.  If invalid, we
 *  set OptionSwitch to point to it and return errflag.
 *
 *  If valid, we then check to see if there should be an extended
 *  arguement.  If there should not be, we set OptionSwitch to NULL
 *  and return the switch character.
 *
 *  If there should be an extended arg, we check for it in two
 *  places: first, following the switch character in the same
 *  arguement.  If not found there, we look for the next arg.  If
 *  not found, set OptionSwitch NULL.  Return the switch character.
 *
 *	Format of valid switches string -- all of the valid switches
 *	lumped together into a single string.  Switches which have
 *	extended arguements must be followed by the extended arg
 *	flag, ':' for default.  Thus "abc:d" would mean the switches
 *	-a, -b, -c, and -d are all legal.  In addition, the -c
 *	switch may have some sort of trailing parameter.
 *
 *  History -- this function was once part of SCS standard library.
 *  It has been moved here rather than carry the whole library along.
 *  There are more functions relating to this, but they have been
 *  removed as irrelevant to this implementation.
 */

#ifdef __STDC__
int	OptionParse( int argc, char *argv[], char *opts )
#else
int	OptionParse( argc, argv, opts )
int	argc ;
char	*argv[] ;
char	*opts ;
#endif
{
	register char	c ;	/* General char holder */
	register char*	cp ;	/* Pointer into options list for this switch */

	OptionSwitch = NULL ;

	/*
	 * If we're just starting to look at this list ( argchar == 1 )
	 * we just check two things -- end of arg list, and switch presence
	 */
	
	if ( argchar == 1 )
	{
		if ( argnum >= argc )	/* At end of arg list? */
			return EOF ;
		if ( argv[ argnum ][ 0 ] != swflag )
		{
			/* No switch flag on arg */
			OptionSwitch = argv[ argnum++ ] ;
			return '\0' ;
		}
		else if ( argv[ argnum ][ 1 ] == '\0' )
		{	/* No arg following switch char */
			/* Return switch as arg, not switch */
			dummy[ 0 ] = argv[ argnum++ ][ 0 ] ;
			OptionSwitch = (char*) dummy ;
			return '\0' ;
		}
	}

	/*
	 * When we're past the first character, it means we are
	 * seeing multiple characters under a single switch, eg, '-TagP'.
	 * This is the equivalent of '-T -a -g -P', unless of course
	 * extended processing is desired.  Thus, if 'T' was a solo
	 * switch but 'a' required extended processing, '-TagP' would
	 * be equivalent to '-T -a gP', where 'gP' is the extended arg
	 * of 'a'.
	 */
	
	c = argv[ argnum ][ argchar ] ;
	/* Check to see if this is a good switch */
	if ( ( cp = strchr( opts, c ) ) == NULL )
	{
		/* Bad switch. */
		if ( argv[ argnum ][ ++argchar ] == '\0' )
		{	/* If end of arg, reset to look at next arg */
			argnum++ ;
			argchar = 1 ;
		}
		/* Build the error return and return it */
		dummy[ 0 ] = c ;
		OptionSwitch = (char*) dummy ;
		return errflg ;
	}

	/*
	 * Check for extended arg with this switch.  Check both
	 * formats, either '-S extarg' (two args) or '-Sextarg' (one arg)
	 */
	
	if ( *++cp == extarg )
	{
		if ( argv[ argnum ][ argchar + 1 ] != '\0' )
			OptionSwitch = &argv[ argnum++ ][ argchar + 1 ] ;
		else if ( ++argnum < argc )	/* Found extended */
			OptionSwitch = argv[ argnum++ ] ;
		argchar = 1 ;	/* Start at beginning of next arg */
	}
	else if ( argv[ argnum ][ ++argchar ] == '\0' )
	{
		/* Arg complete -- reset for next arg */
		argchar = 1 ;
		argnum++ ;
	}
	return c ;
}



/*
 * Print a usage message.  If the specified value is non-negative, exit
 * with that value.
 */

#ifdef	__STDC__
static void	usage( int exitval )
#else
static void	usage( exitval )
int	exitval ;
#endif
{
	(void) fprintf( stderr, "Usage: %s [ -v ] [ -h ] [ -s system ]\n", ProgName ) ;
	(void) fprintf( stderr, "\tThe `-v' flag will print version information.\n" ) ;
	(void) fprintf( stderr, "\tThe `-h' flag will print this information and quit.\n" ) ;
	(void) fprintf( stderr, "\tThe `-s' flag will select a system to examine.\n" ) ;
	if ( exitval >= 0 )
		exit( exitval ) ;
	return ;
}


/*
 * Do the report for a given system
 */

#ifdef	__STDC__
void	examine_pending( char *name )
#else
void	examine_pending( name )
char	*name ;
#endif
{
	int	first1 = 1 ;	/* flag for 1st time thru */
	FILE	*lsout ;
	FILE	*xfile ;
	char	filename[ MAXNAMELEN + 1 ] ;
	char	xfilename[ MAXNAMELEN + 1 ] ;
	char	dataline[ BUFSIZ + 1 ] ;
	char	errbuf[ BUFSIZ ] ;
	int	slen ;		/* string length */

	/*
	 *  This gives us a prealphabetized list of the pending
	 *  execution files.  It beats the heck out of writing code
	 *  to open, read, and sort a directory.
	 */
	lsout = popen( "ls X.* 2>&1", "r" ) ;
	if ( lsout == NULL )
	{
		(void) perror( "Can't start ls" ) ;
		exit( 1 ) ;
	}

	filename[ MAXNAMELEN ] = EOS ;
	/* loop checking files */
	while ( fgets( filename,  MAXNAMELEN, lsout ) )
	{
		if ( strlen( filename ) > MAXNAMELEN )
		{
			(void) fprintf( stderr, "The filename `%s' is too long, skipping.\n, filename" ) ;
			break ;
		}

		/* see if there's a file found */
		(void) sscanf( filename, "%s", xfilename ) ;
		if ( strncmp( xfilename, "X.*", 3 ) == 0 )
		{
			(void) printf( "No actions pending for `%s'.\n", name ) ;
			break ;
		}

		/* open the file for reading */
		xfile = fopen( xfilename, "r" ) ;
		if ( xfile == NULL )
		{
			(void) sprintf( errbuf, "Can't open info file `%s' for system `%s'.\n", xfilename, name ) ;
			(void) perror( errbuf ) ;
			break ;
		}

		/* check for header needed */
		if ( first1 )
		{
			first1 = 0 ;
			(void) printf( "\n %-15s %-15s %-15s %s\n",
				"Job", "User", "Input File", "Task" ) ;
		}

		/* scan the file for various things */
		(void) printf( "%-15s ", xfilename + 2 ) ;
		dataline[ MAXNAMELEN ] = EOS ;
		while ( fgets( dataline,  MAXNAMELEN, xfile ) )
		{
			if ( ( slen = strlen( dataline ) ) > ( MAXNAMELEN -1 ) )
			{
				/* line too long */
				while ( getc( xfile ) != '\n' ) ;
			}
			dataline[slen - 1] = EOS ;

			switch ( dataline[0] )
			{
			case 'U':	/* originating user */
				(void) printf( "%-15s ", dataline + 2 ) ;
				break ;
			case 'C':	/* command */
				(void) printf( "%s\n", dataline + 2 ) ;
				break ;
			case 'I':	/* input file */
				(void) printf( "%-15s ", dataline + 2 ) ;
				break ;
			}
		}

		/* clean up */
		(void) fclose( xfile ) ;
	}
	(void) pclose( lsout ) ;
}


#ifdef	__STDC__
int	main( int argc, char *argv[] )
#else
main( argc, argv )
int	argc ;
char	*argv[] ;
#endif
{
	int	switchchar ;
	int	list_index = 0 ;
	char	**system ;

	/* test input */

	ProgName = argv[ 0 ] ;
	if ( argc < 2 )
		usage( 1 ) ;
        if ( argc < 2 )
	{
		(void) fprintf( stderr, "%s: You must specify something to do!\n", ProgName ) ;
		usage( 1 ) ;
	}
	while ( EOF != ( switchchar = OptionParse( argc, argv, "hvs:" ) ) )
	{
		switch( switchchar )
		{
		  case 's':
			if ( list_index == BUFSIZ )
			{
				(void) fprintf( stderr, "%s: Sorry, you cannot specify more than %d systems.\n", ProgName, BUFSIZ ) ;
				exit( 1 ) ;
			}
			SystemList[ list_index++ ] = NewString( OptionSwitch ) ;
			break ;
		  case 'v':
			version_flag = TRUE ;
			break ;
		  case 'h':
			help_flag = TRUE ;
			break ;
		  default:
			(void) fprintf( stderr, "%s: No such switch as `-%c'.\n", ProgName, *OptionSwitch ) ;
			usage( 1 ) ;
			break ;
		}
	}
	SystemList[ list_index ] = NULL ;

	if ( version_flag )
	{
		char	**msgptr ;
		for ( msgptr = IDent ; *msgptr != NULL ; msgptr++ )
			(void) printf( "%s\n", *msgptr ) ;
	}
	if ( help_flag )
	{
		usage( -1 ) ;
	}
	/*
	 * There was at least one valid switch, but did they ask for
	 * a system?  If not, exit quietly
	 */
	if ( list_index == 0 )
		exit( 0 ) ;
	
	if ( 0 != chdir( SPOOLDIR ) )
	{
		(void) fprintf( stderr, "%s: could not access spool directory `%s'.\n", ProgName, SPOOLDIR ) ;
		exit( 1 ) ;
	}
	for ( system = SystemList ; *system != NULL ; system++ )
	{
		if ( 0 != chdir( *system ) )
			(void) fprintf( stderr, "%s: no such system as `%s'.\n", ProgName, *system ) ;
		else
		{
			examine_pending( *system ) ;
			(void) chdir( ".." ) ;
		}
	}
	exit( 0 ) ;
	/* next two lines are to silence lint and gcc -Wall */
	/* NOTREACHED */
	return 0 ;
}
